import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod } from '@nestjs/microservices';
import { RpcValidationPipe } from 'src/modules/utils/pipes/rpc.validation-pipe';
import { MatchRequest } from '../../domain/dtos/match.request';
import { ICollection } from 'src/modules/database/src/interfaces/collection.interface';
import { Match } from '../../domain/entities/match';
import { MatchQuery } from '../../queries/match.query';
import { MatchPresenter } from '../secondaries/match.presenter';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class MatcherController {
  constructor(
    private readonly _queryBus: QueryBus,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  @GrpcMethod('MatcherService', 'Match')
  async match(data: MatchRequest): Promise<ICollection<Match>> {
    const matchCollection = await this._queryBus.execute(new MatchQuery(data));
    return Promise.resolve({
      data: matchCollection.data.map((match: Match) =>
        this._mapper.map(match, Match, MatchPresenter),
      ),
      total: matchCollection.total,
    });
  }
}
