import {
  IsArray,
  IsBoolean,
  IsDate,
  IsEnum,
  IsInt,
  IsNumber,
  IsOptional,
  Max,
  Min,
} from 'class-validator';
import { AutoMap } from '@automapper/classes';
import { Point } from '../entities/point.type';
import { Schedule } from '../entities/schedule.type';
import { MarginDurations } from '../entities/margin_durations.type';
import { Algorithm } from './algorithm.enum';

export class MatchRequest {
  @IsArray()
  @AutoMap()
  waypoints: Array<Point>;

  @IsDate()
  @IsOptional()
  @AutoMap()
  departure: Date;

  @IsDate()
  @IsOptional()
  @AutoMap()
  fromDate: Date;

  @IsOptional()
  @AutoMap()
  schedule: Schedule;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver: boolean;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger: boolean;

  @IsOptional()
  @IsDate()
  @AutoMap()
  toDate: Date;

  @IsOptional()
  @IsInt()
  @AutoMap()
  marginDuration: number;

  @IsOptional()
  @AutoMap()
  marginDurations: MarginDurations;

  @IsOptional()
  @IsNumber()
  @AutoMap()
  seatsPassenger: number;

  @IsOptional()
  @IsNumber()
  @AutoMap()
  seatsDriver: number;

  @IsOptional()
  @AutoMap()
  strict: boolean;

  @IsOptional()
  @IsEnum(Algorithm)
  @AutoMap()
  algorithm: Algorithm;

  @IsOptional()
  @IsNumber()
  @AutoMap()
  remoteness: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  useProportion: boolean;

  @IsOptional()
  @IsNumber()
  @Min(0)
  @Max(1)
  @AutoMap()
  proportion: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  useAzimuth: boolean;

  @IsOptional()
  @IsInt()
  @Min(0)
  @Max(359)
  @AutoMap()
  azimuthMargin: number;

  @IsOptional()
  @IsNumber()
  @Min(0)
  @Max(1)
  @AutoMap()
  maxDetourDistanceRatio: number;

  @IsOptional()
  @IsNumber()
  @Min(0)
  @Max(1)
  @AutoMap()
  maxDetourDurationRatio: number;

  @IsOptional()
  @IsArray()
  exclusions: Array<number>;
}
